/**
 * Copyright (c) Members of the EGEE Collaboration. 2007-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2007-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *
 */

#include <unistd.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>

#include "readbuf.h"
#include "scas_log.h"

ssize_t readfdtobuf (int fd, char ** buffer, size_t buffersize)
{
    ssize_t num_of_bytes = 0;
    struct stat filestat;
    int myerrno;

    if (fd < 0)
    {
        fprintf(stderr,
		"%s: Error: file descriptor is negative.\n", __func__);
        scas_log(LOG_ERR,
		"%s: Error: file descriptor is negative.\n", __func__);
        return -11;
    }


    if ((!buffer) || (!*buffer))
    {
        fprintf(stderr, "%s: Error: buffer not provided or set.\n", __func__);
        scas_log(LOG_ERR, "%s: Error: buffer not provided or set.\n", __func__);
        return -12;
    }


    /* How big is that file again? */
    if (fstat(fd, &filestat))
    {
	myerrno=errno;
	fprintf(stderr, "%s: stat on the opened file failed: %s\n",
		__func__, strerror(myerrno));
	scas_log(LOG_ERR, "%s: stat on the opened file failed: %s\n",
		__func__, strerror(myerrno));
        return -51;
    }


    /* 
     * Read its contents into the allocated buffer in memory while making
     * sure the amound read is what we expect it to be... 
     */
    num_of_bytes = read(fd, *buffer, buffersize);
    if (filestat.st_size != num_of_bytes) 
    {
        fprintf(stderr,
		"%s: Read wrong number of bytes while reading the source proxy. "
	        "Read %ld bytes instead of the expected %lu bytes\n",
		 __func__, (long)num_of_bytes, (long unsigned)buffersize);
        scas_log(LOG_ERR,
		 "%s: Read wrong number of bytes while reading the source proxy. "
	         "Read %ld bytes instead of the expected %lu bytes\n",
		 __func__, (long)num_of_bytes, (long unsigned)buffersize);
        return -91;
    }

    /* Need to store the size for later use during writing of the proxy. */
    return num_of_bytes;
}


